/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.connector.pdr;

import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.util.EcoreUtil;

import tools.descartes.dql.connector.AbstractOSGiQueryConnector;
import tools.descartes.dql.connector.ModelStructureQueryConnector;
import tools.descartes.dql.connector.QueryProgressListener;
import tools.descartes.dql.core.engine.util.DQLLogger;
import tools.descartes.dql.models.mapping.domain.DomainFactory;
import tools.descartes.dql.models.mapping.domain.Entity;
import tools.descartes.dql.models.mapping.domain.Probe;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;
import tools.descartes.dql.models.repository.Observation;
import tools.descartes.dql.models.repository.ObservationRepository;

public class ModelStructureQueryConnectorImpl extends AbstractOSGiQueryConnector
		implements ModelStructureQueryConnector {
	private final Logger log = DQLLogger.getLogger(this.getClass().getName());
	private final DomainFactory domainFactory = DomainFactory.eINSTANCE;

	@Override
	public EntityMapping requestDoF(final EntityMapping request) {
		final EntityMapping response = EcoreUtil.copy(request);
		final String modelLocation = RepositoryHelper.qualifyModelLocation(request.getModelLocation());

		// open model
		final ObservationRepository repository = RepositoryHelper.loadModelInstance(modelLocation);

		// iterate
		final Observation latestObservation = repository.getLatestObservation();
		final EntityMapping latest = latestObservation.getObservation();

		// copy collections
		response.getDoF().addAll(EcoreUtil.copyAll(latest.getDoF()));

		return response;
	}

	@Override
	public EntityMapping requestEntities(final EntityMapping request) {
		final EntityMapping response = EcoreUtil.copy(request);
		final String modelLocation = RepositoryHelper.qualifyModelLocation(request.getModelLocation());

		// open model
		final ObservationRepository repository = RepositoryHelper.loadModelInstance(modelLocation);

		// iterate
		final Observation latestObservation = repository.getLatestObservation();
		final EntityMapping latest = latestObservation.getObservation();

		for (final Entity r : latest.getEntities()) {
			final Entity copy = EcoreUtil.copy(r);

			copy.getProbes().clear();

			response.getEntities().add(copy);
		}

		return response;
	}

	@Override
	public EntityMapping requestProbes(final EntityMapping request) {
		final EntityMapping response = EcoreUtil.copy(request);
		final String modelLocation = RepositoryHelper.qualifyModelLocation(request.getModelLocation());

		// open model
		final ObservationRepository repository = RepositoryHelper.loadModelInstance(modelLocation);

		// iterate
		final Observation latestObservation = repository.getLatestObservation();
		final EntityMapping latest = latestObservation.getObservation();
		final Map<String, Entity> latestEntityIndex = RepositoryHelper.buildEntityIndex(latest);

		for (final Entity e : response.getEntities()) {
			final String identifier = e.getIdentifier();
			final Entity latestEntity = latestEntityIndex.get(identifier);

			if (latestEntity == null) {
				log.warn("No entity found for identifier " + identifier);
				continue;
			}

			for (final Probe latestProbe : latestEntity.getProbes()) {
				final Probe responseProbe = domainFactory.createProbe();
				final EList<Probe> probes = e.getProbes();

				responseProbe.setMetricName(latestProbe.getMetricName());
				probes.add(responseProbe);
			}
		}

		return response;
	}

	@Override
	public boolean supportsDoFRequests() {
		return true;
	}

	@Override
	public boolean supportsEntityRequests() {
		return true;
	}

	@Override
	public boolean supportsProbeRequests() {
		return true;
	}

	@Override
	public void addQueryProgressListener(QueryProgressListener listener) {

	}
}
