package tools.descartes.dql.connector.vsp;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.emf.ecore.util.EcoreUtil;

import tools.descartes.dql.connector.AbstractOSGiQueryConnector;
import tools.descartes.dql.connector.ModelStructureQueryConnector;
import tools.descartes.dql.connector.QueryProgressListener;
import tools.descartes.dql.connector.vsp.reuse.VSphereConnectionHelper;
import tools.descartes.dql.core.engine.util.DQLLogger;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;

import com.vmware.vim25.ManagedObjectReference;
import com.vmware.vim25.ObjectSpec;
import com.vmware.vim25.PropertyFilterSpec;
import com.vmware.vim25.PropertySpec;
import com.vmware.vim25.RetrieveOptions;
import com.vmware.vim25.RetrieveResult;
import com.vmware.vim25.SelectionSpec;
import com.vmware.vim25.ServiceContent;
import com.vmware.vim25.TraversalSpec;
import com.vmware.vim25.VimPortType;

public class ModelStructureQueryConnectorImpl extends
		AbstractOSGiQueryConnector implements ModelStructureQueryConnector {
	private final Logger log = DQLLogger.getLogger(this.getClass().getName());
	
	
	@Override
	public String getBundleProperty(final String key) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public EntityMapping requestDoF(final EntityMapping request) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public EntityMapping requestEntities(final EntityMapping request)
			throws RuntimeException {
		final EntityMapping response = EcoreUtil.copy(request);
		final VSphereSession session = new VSphereSession();
		final IWorkspaceRoot root = VSphereHelper.getWorkspaceRoot();
		final String modelLocation = request.getModelLocation();
		final Properties properties = VSphereHelper.loadModelProperties(root,
				modelLocation);
		final String url = properties
				.getProperty(VSphereHelper.CONNECTION_URL_KEY);
		final String username = properties
				.getProperty(VSphereHelper.CONNECTION_USERNAME_KEY);
		final String password = properties
				.getProperty(VSphereHelper.CONNECTION_PASSWORD_KEY);

		VSphereConnectionHelper.connect(session, url, username, password);

		final ServiceContent content = session.getServiceContent();
		final ManagedObjectReference perfManagerReference = content
				.getPerfManager();
		final VimPortType vimPort = session.getVimPort();

		RetrieveResult result = null;

		List<PropertyFilterSpec> propertyFilterSpecs = new ArrayList<PropertyFilterSpec>(
				1);
		createPropertyFilterSpec(propertyFilterSpecs, content.getRootFolder());

		try {
			result = vimPort.retrievePropertiesEx(
					content.getPropertyCollector(), propertyFilterSpecs,
					new RetrieveOptions());
		} catch (Exception e1) {
			log.error("Retrieving available properties failed: "
					+ e1.getMessage());
			e1.printStackTrace();
		}

		System.out.println(result);

		VSphereConnectionHelper.disconnect(session);

		return response;
	}

	private void createPropertyFilterSpec(
			List<PropertyFilterSpec> propertyFilterSpecs,
			ManagedObjectReference rootFolder) {
		PropertyFilterSpec pfs = new PropertyFilterSpec();
		propertyFilterSpecs.add(pfs);

		// Create Property Spec
		PropertySpec propertySpec = new PropertySpec();
		propertySpec.setAll(Boolean.TRUE);
		propertySpec.setType("VirtualMachine");
		pfs.getPropSet().add(propertySpec);

		// Now create Object Spec
		ObjectSpec objectSpec = new ObjectSpec();
		objectSpec.setObj(rootFolder);
		objectSpec.setSkip(Boolean.TRUE);
		objectSpec.getSelectSet().add(getVMTraversalSpec());
		pfs.getObjectSet().add(objectSpec);
	}

	@Override
	public EntityMapping requestProbes(final EntityMapping request) {
		EntityMapping response = EcoreUtil.copy(request);

		return response;
	}

	@Override
	public boolean supportsDoFRequests() {
		return false;
	}

	@Override
	public boolean supportsEntityRequests() {
		return true;
	}

	@Override
	public boolean supportsProbeRequests() {
		return true;
	}

	private static TraversalSpec getVMTraversalSpec() {
		// Create a traversal spec that starts from the 'root' objects
		// and traverses the inventory tree to get to the VirtualMachines.
		// Build the traversal specs bottoms up

		// Traversal to get to the VM in a VApp
		TraversalSpec vAppToVM = new TraversalSpec();
		vAppToVM.setName("vAppToVM");
		vAppToVM.setType("VirtualApp");
		vAppToVM.setPath("vm");

		// Traversal spec for VApp to VApp
		TraversalSpec vAppToVApp = new TraversalSpec();
		vAppToVApp.setName("vAppToVApp");
		vAppToVApp.setType("VirtualApp");
		vAppToVApp.setPath("resourcePool");
		// SelectionSpec for VApp to VApp recursion
		SelectionSpec vAppRecursion = new SelectionSpec();
		vAppRecursion.setName("vAppToVApp");
		// SelectionSpec to get to a VM in the VApp
		SelectionSpec vmInVApp = new SelectionSpec();
		vmInVApp.setName("vAppToVM");
		// SelectionSpec for both VApp to VApp and VApp to VM
		List<SelectionSpec> vAppToVMSS = new ArrayList<SelectionSpec>();
		vAppToVMSS.add(vAppRecursion);
		vAppToVMSS.add(vmInVApp);
		vAppToVApp.getSelectSet().addAll(vAppToVMSS);

		// This SelectionSpec is used for recursion for Folder recursion
		SelectionSpec sSpec = new SelectionSpec();
		sSpec.setName("VisitFolders");

		// Traversal to get to the vmFolder from DataCenter
		TraversalSpec dataCenterToVMFolder = new TraversalSpec();
		dataCenterToVMFolder.setName("DataCenterToVMFolder");
		dataCenterToVMFolder.setType("Datacenter");
		dataCenterToVMFolder.setPath("vmFolder");
		dataCenterToVMFolder.setSkip(false);
		dataCenterToVMFolder.getSelectSet().add(sSpec);

		// TraversalSpec to get to the DataCenter from rootFolder
		TraversalSpec traversalSpec = new TraversalSpec();
		traversalSpec.setName("VisitFolders");
		traversalSpec.setType("Folder");
		traversalSpec.setPath("childEntity");
		traversalSpec.setSkip(false);
		List<SelectionSpec> sSpecArr = new ArrayList<SelectionSpec>();
		sSpecArr.add(sSpec);
		sSpecArr.add(dataCenterToVMFolder);
		sSpecArr.add(vAppToVM);
		sSpecArr.add(vAppToVApp);
		traversalSpec.getSelectSet().addAll(sSpecArr);
		return traversalSpec;
	}
	
	public void addQueryProgressListener(QueryProgressListener listener)
	{
		
	}
}
