/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.core.engine.interpreter;

import java.util.List;

import tools.descartes.dql.models.mapping.mapping.EntityMapping;

/**
 * Interface defining the interpretation of Xtext's incoming model and
 * transforms the model into instances of the Mapping Meta-Model, executes the
 * query and stores the result after query execution.
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 */
public interface QueryInterpreter {
	/**
	 * Returns the first result or in case of a single result set, the only
	 * result.
	 * 
	 * @return The first result of the query execution
	 */
	public EntityMapping getResult();

	/**
	 * Returns all available query results.
	 * 
	 * @return All available results of the query execution
	 */
	public List<EntityMapping> getResults();

	/**
	 * Returns if the interpretation or execution of the query led to multiple
	 * results.
	 * 
	 * @return True, if multiple results are available, false otherwise
	 */
	public boolean hasMultipleResults();

	/**
	 * Starts the interpretation process of an Xtext processed model instance.
	 * 
	 * If the interpretation was successful, the results can be retrieved
	 * through the available getter methods.
	 * 
	 * @see #getResult()
	 * @see #getResults()
	 * 
	 * @return True, if the validation and interpretation was executed
	 *         successfully, otherwise false
	 */
	public boolean interpretQuery();
}
