/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.core.platform.eclipse.launch;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

public class CancelableEngineJob extends Job {
	public static final String DQL_JOB_NAME = "DQL Query Execution Engine";

	private IResource resource;
	private RuntimeException e = null;
	
	public CancelableEngineJob(IResource resource) {
		super(DQL_JOB_NAME);
		super.setPriority(Job.LONG);
		super.setUser(true);
		this.resource = resource;
	}

	@Override
	protected IStatus run(IProgressMonitor monitor) {
        Thread t = new Thread(new QueryExecutionEngineThread(resource, monitor, this));
        
        t.start();
        
        while (!monitor.isCanceled() && t.isAlive()) {
        	try { Thread.sleep(500); } catch (InterruptedException e) {}
        	if (e != null)
        		throw e;
        }
        
		return Status.OK_STATUS;
	}

	public void throwException(RuntimeException e) {
		this.e = e;
	}
}
