/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.core.platform.eclipse.launch;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.debug.ui.ILaunchShortcut;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.ui.IEditorPart;
import org.eclipse.xtext.ui.editor.XtextEditor;

/**
 * This class is registered as LaunchShortcut, i.e. it is made available as
 * '[Right click on a resource] -> Run As' using an extension point defined in
 * the plug-in configuration
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 */
public class LaunchShortcut implements ILaunchShortcut {

	private void execute(final IResource resource) {
		new CancelableEngineJob(resource).schedule();
	}

	@Override
	public void launch(final IEditorPart editor, final String mode) {
		final XtextEditor xtextEditor = (XtextEditor) editor;
		final IResource resource = xtextEditor.getResource();

		execute(resource);
	}

	/**
	 * Wraps the incoming ISelection into an ITreeSelection into an IAdaptable
	 * into an IResource. IResource is the reference to the selected element
	 * from the workspace. As multiple elements might have been selected, the
	 * launcher iterates over all selected elements.
	 */
	@Override
	public void launch(final ISelection selection, final String mode) {

		final ITreeSelection treeSelection = (ITreeSelection) selection;
		final TreePath[] selectedElements = treeSelection.getPaths();

		for (final TreePath element : selectedElements) {
			IResource resource;
			if (element.getFirstSegment() instanceof IAdaptable) {
				final IAdaptable adaptable = (IAdaptable) element.getLastSegment();
				resource = (IResource) adaptable.getAdapter(IResource.class);

				execute(resource);
			}
		}
	}
}
