/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.lang.serializer;

import com.google.inject.Inject;
import java.util.List;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.IGrammarAccess;
import org.eclipse.xtext.RuleCall;
import org.eclipse.xtext.nodemodel.INode;
import org.eclipse.xtext.serializer.analysis.GrammarAlias.AbstractElementAlias;
import org.eclipse.xtext.serializer.analysis.GrammarAlias.TokenAlias;
import org.eclipse.xtext.serializer.analysis.ISyntacticSequencerPDAProvider.ISynNavigable;
import org.eclipse.xtext.serializer.analysis.ISyntacticSequencerPDAProvider.ISynTransition;
import org.eclipse.xtext.serializer.sequencer.AbstractSyntacticSequencer;
import tools.descartes.dql.lang.services.DescartesQLGrammarAccess;

@SuppressWarnings("all")
public class DescartesQLSyntacticSequencer extends AbstractSyntacticSequencer {

	protected DescartesQLGrammarAccess grammarAccess;
	protected AbstractElementAlias match_WhatIfValueVariationClause_CommaKeyword_2_0_q;
	
	@Inject
	protected void init(IGrammarAccess access) {
		grammarAccess = (DescartesQLGrammarAccess) access;
		match_WhatIfValueVariationClause_CommaKeyword_2_0_q = new TokenAlias(false, true, grammarAccess.getWhatIfValueVariationClauseAccess().getCommaKeyword_2_0());
	}
	
	@Override
	protected String getUnassignedRuleCallToken(EObject semanticObject, RuleCall ruleCall, INode node) {
		return "";
	}
	
	
	@Override
	protected void emitUnassignedTokens(EObject semanticObject, ISynTransition transition, INode fromNode, INode toNode) {
		if (transition.getAmbiguousSyntaxes().isEmpty()) return;
		List<INode> transitionNodes = collectNodes(fromNode, toNode);
		for (AbstractElementAlias syntax : transition.getAmbiguousSyntaxes()) {
			List<INode> syntaxNodes = getNodesFor(transitionNodes, syntax);
			if (match_WhatIfValueVariationClause_CommaKeyword_2_0_q.equals(syntax))
				emit_WhatIfValueVariationClause_CommaKeyword_2_0_q(semanticObject, getLastNavigableState(), syntaxNodes);
			else acceptNodes(getLastNavigableState(), syntaxNodes);
		}
	}

	/**
	 * Ambiguous syntax:
	 *     ','?
	 *
	 * This ambiguous syntax occurs at:
	 *     variationClause+=ArithmeticClause (ambiguity) variationClause+=ArithmeticClause
	 *     variationClause+=ArithmeticClause (ambiguity) variationClause+=IntervalVariationClause
	 *     variationClause+=ArithmeticClause (ambiguity) variationClause+=ValueVariationClause
	 *     variationClause+=ArithmeticClause (ambiguity) variationClause+=VariationInstruction
	 *     variationClause+=IntervalVariationClause (ambiguity) variationClause+=ArithmeticClause
	 *     variationClause+=IntervalVariationClause (ambiguity) variationClause+=IntervalVariationClause
	 *     variationClause+=IntervalVariationClause (ambiguity) variationClause+=ValueVariationClause
	 *     variationClause+=IntervalVariationClause (ambiguity) variationClause+=VariationInstruction
	 *     variationClause+=ValueVariationClause (ambiguity) variationClause+=ArithmeticClause
	 *     variationClause+=ValueVariationClause (ambiguity) variationClause+=IntervalVariationClause
	 *     variationClause+=ValueVariationClause (ambiguity) variationClause+=ValueVariationClause
	 *     variationClause+=ValueVariationClause (ambiguity) variationClause+=VariationInstruction
	 *     variationClause+=VariationInstruction (ambiguity) variationClause+=ArithmeticClause
	 *     variationClause+=VariationInstruction (ambiguity) variationClause+=IntervalVariationClause
	 *     variationClause+=VariationInstruction (ambiguity) variationClause+=ValueVariationClause
	 *     variationClause+=VariationInstruction (ambiguity) variationClause+=VariationInstruction
	 */
	protected void emit_WhatIfValueVariationClause_CommaKeyword_2_0_q(EObject semanticObject, ISynNavigable transition, List<INode> nodes) {
		acceptNodes(transition, nodes);
	}
	
}
