package tools.descartes.pavo;

/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;

import org.jfree.ui.RefineryUtilities;

import tools.descartes.dql.models.mapping.domain.DoF;
import tools.descartes.dql.models.mapping.domain.Entity;
import tools.descartes.dql.models.mapping.domain.Probe;
import tools.descartes.dql.models.mapping.domain.StatType;
import tools.descartes.dql.models.mapping.domain.StringResult;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;
import tools.descartes.pavo.controller.TransformEntityMappingToResult;
import tools.descartes.pavo.controller.VisualizationController;
import tools.descartes.pavo.result.ResultContainerPavo;
import tools.descartes.pavo.util.StatTypeAndDof;
import tools.descartes.pavo.util.dof;

/**
 * Main class of the PAVO library. Creates one frame with one Tab for each resource received by the EntityMapping just create an object of this class with the EntityMapping and a title, to run PAVO.
 * The frame constits out of the query text below the tab bar. Beneith the Query there is the panel received by each VisualizationController
 * 
 * @author Maximilian König <max.c.koenig@t-online.de>
 * @since 2016
 * @see VisualizationController
 */

public class GraphicsEngine {

	private JFrame mainFrame = new JFrame();
	private Collection<EntityMapping> currentMapping;
	private JTabbedPane tabbedPane = new JTabbedPane();

	public GraphicsEngine(String title, EntityMapping mappingNeu) {

		currentMapping = new LinkedList<EntityMapping>();
		currentMapping.add(mappingNeu);
		mainFrame.setTitle(title);
		mainFrame.setBackground(Color.WHITE);
		mainFrame.add(tabbedPane);
		interpreteEntityMapping(currentMapping, tabbedPane);
		mainFrame.pack();
		mainFrame.setMinimumSize(new Dimension(mainFrame.getWidth() + 100, mainFrame.getHeight()));
		mainFrame.setResizable(false);
		mainFrame.setVisible(true);
		mainFrame.toFront();
		mainFrame.setMaximumSize(new Dimension(1000, 1000));

		RefineryUtilities.centerFrameOnScreen(mainFrame);
	}

	public GraphicsEngine(String title, Collection<EntityMapping> mappinglist) {

		currentMapping = mappinglist;
		mainFrame.setTitle(title);
		mainFrame.setBackground(Color.WHITE);
		mainFrame.add(tabbedPane);
		interpreteEntityMapping(currentMapping, tabbedPane);
		mainFrame.pack();
		mainFrame.setMinimumSize(new Dimension(mainFrame.getWidth() + 100, mainFrame.getHeight()));
		mainFrame.setResizable(false);
		mainFrame.setVisible(true);
		mainFrame.toFront();
		mainFrame.setMaximumSize(new Dimension(1000, 1000));

		RefineryUtilities.centerFrameOnScreen(mainFrame);
	}



	/**
	 * Method to interprete the EntityMapping received by DQL and merges
	 * diagrams creates tabs for each resource/entities for this resource
	 * 
	 * @param EntityMapping
	 *            mapping: current EntityMapping to be visualized
	 * @param JTabbedPane
	 *            tabPane: Tab Pane for all different resources.
	 * 
	 */
	public static void interpreteEntityMapping(Collection<EntityMapping> mapping, JTabbedPane tabPane) {

		List<String> metricNames = new LinkedList<String>();
		List<List<String>> metricNamesDescription = new LinkedList<List<String>>();
		List<List<String>> resourceNames = new LinkedList<List<String>>();
		List<List<StatTypeAndDof>> listStatTypeToMetric = new LinkedList<List<StatTypeAndDof>>();
		List<List<List<dof>>> doftoStatType = new LinkedList<List<List<dof>>>();

		for (EntityMapping m : mapping) {
			List<dof> dofs = new LinkedList<dof>();
			for (DoF s : m.getDoF()) {
				dofs.add(new dof(s.getIdentifier(),
						((StringResult) s.getProbes().get(0).getStatTypes().get(0).getResult()).getValue()));
			}

			for (Entity e : m.getEntities()) {

				for (Probe p : e.getProbes()) {

					for (StatType s : p.getStatTypes()) {

						boolean found = false;
						String newName = p.getMetricName();
						String newName2;
						if(m.getDescription()==null || m.getDescription().equals(null))
							newName2 = p.getMetricName();
						else
							newName2 = p.getMetricName() + " " + m.getDescription();
						
						for (int i = 0; i < metricNames.size(); i++) {

							if (metricNames.get(i).equals(newName)) {
								metricNamesDescription.get(i).add(newName2);
								listStatTypeToMetric.get(i).add(new StatTypeAndDof(s, dofs));
								resourceNames.get(i).add(e.getAlias());
								found = true;
							}
						}
						if (found == false) {
							metricNames.add(newName);
							LinkedList<StatTypeAndDof> help = new LinkedList<StatTypeAndDof>();
							help.add(new StatTypeAndDof(s, dofs));
							LinkedList<String> help2 = new LinkedList<String>();
							help2.add(e.getAlias());
							listStatTypeToMetric.add(help);
							resourceNames.add(help2);
							LinkedList<String> help3 = new LinkedList<String>();
							help3.add(newName2);
							metricNamesDescription.add(help3);
						}
					}
				}
			}
		}

		for (int i = 0; i < metricNames.size(); i++) {

			tabPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
			String queryText = ((EntityMapping) ((List<EntityMapping>) mapping).get(0)).getQueryAsString();
			JTextArea querytext = new JTextArea(queryText);
			querytext.setFont(new Font("Arial", Font.BOLD, 15));
			querytext.setBackground(Color.WHITE);
			querytext.setOpaque(true);
			querytext.setSize(new Dimension(tabPane.getWidth(), 200));
			JPanel resourcePanel = new JPanel(new BorderLayout());
			resourcePanel.add(querytext, BorderLayout.NORTH);
			ResultContainerPavo resultContainer = TransformEntityMappingToResult
					.transformEntityMapping(listStatTypeToMetric.get(i), resourceNames.get(i), metricNames.get(i),
							metricNamesDescription.get(i));
			VisualizationController engine = new VisualizationController(resultContainer);

			resourcePanel.add(Box.createRigidArea(new Dimension(tabPane.getWidth(), 20)), BorderLayout.EAST);
			resourcePanel.add(engine.getView().getCompletePanel(), BorderLayout.SOUTH);
			tabPane.addTab(metricNames.get(i), resourcePanel);
		}

	}
	
}

