/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pavo.controller;


import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.EList;

import tools.descartes.dql.models.mapping.domain.AbstractSeriesResult;
import tools.descartes.dql.models.mapping.domain.ContinuousSeriesResult;
import tools.descartes.dql.models.mapping.domain.DecimalResult;
import tools.descartes.dql.models.mapping.domain.DiscreteSeriesResult;
import tools.descartes.dql.models.mapping.domain.OnlineResult;
import tools.descartes.dql.models.mapping.domain.Result;
import tools.descartes.dql.models.mapping.domain.ResultList;
import tools.descartes.dql.models.mapping.domain.SeriesResultElement;
import tools.descartes.dql.models.mapping.domain.StringResult;
import tools.descartes.pavo.result.AbstractQuantitativeResult;
import tools.descartes.pavo.result.ResultContainerPavo;
import tools.descartes.pavo.result.SeriesResultPavo;
import tools.descartes.pavo.result.StringResultPavo;
import tools.descartes.pavo.result.ValueResultPavo;
import tools.descartes.pavo.util.PavoLogger;
import tools.descartes.pavo.util.Point;
import tools.descartes.pavo.util.StatTypeAndDof;

/**
 * class to transform the DQL Type Entity Mapping to the PAVO intern type ResultContainerPavo
 * 
 * 
 * @author Maximilian König <max.c.koenig@t-online.de>
 * @since 2016
 */
public class TransformEntityMappingToResult {
	private static Logger log = PavoLogger.getLogger(TransformEntityMappingToResult.class.getName());


	/**
	 * @param currentProbes List of all Probes to show 
	 * @param resourcesToProbes List of all Ressources 	appropriate to the probes (e.g. currentProbes.first has the ressource resourcesToProbes.first())
	 * @param metricName Name of the Metrik
	 * @return
	 */
	public static ResultContainerPavo transformEntityMapping(List<StatTypeAndDof> currentProbes,
			List<String> resourcesToProbes, String metricName, List<String> metricDescription) {
		ResultContainerPavo resultContainer = new ResultContainerPavo();
		
		int j = 0;
		for (StatTypeAndDof currentProbe : currentProbes) {
			
			AbstractQuantitativeResult r = null;
				
			if (currentProbe.getStattype().getResult() instanceof DecimalResult) {
				double value;
				if (((DecimalResult) currentProbe.getStattype().getResult()).getValue() == null) {
					value = 0d;
					log.error("Error parsing result files");
				} else {
					value = ((DecimalResult) currentProbe.getStattype().getResult()).getValue().doubleValue();
				}
				r = new ValueResultPavo(resourcesToProbes.get(j), metricName, metricDescription.get(j),
						currentProbe.getStattype().getTypeName(),
						""
						, "", value, currentProbe.getDofs());
				resultContainer.addResult(r);

			} else if (currentProbe.getStattype().getResult() instanceof StringResult) {

				r = new StringResultPavo(resourcesToProbes.get(j), metricName, metricDescription.get(j),
						currentProbe.getStattype().getTypeName(),
						((StringResult) currentProbe.getStattype().getResult()).getValue().toString());
				resultContainer.addResult(r);
			}

			else if (currentProbe.getStattype().getResult() instanceof DiscreteSeriesResult) {
				
				
				EList<SeriesResultElement> list = ((DiscreteSeriesResult) currentProbe.getStattype().getResult()).getElements();
				List<Point> values = new ArrayList<Point>();

				for (SeriesResultElement a : list) {
					values.add(new Point(a.getX().doubleValue(), a.getY().doubleValue(), a.getConfidenceInterval()));
				}


				r = new SeriesResultPavo(resourcesToProbes.get(j), metricName, metricDescription.get(j),
						currentProbe.getStattype().getTypeName(),
						((AbstractSeriesResult) currentProbe.getStattype().getResult()).getXAxisDescription(),
						((AbstractSeriesResult) currentProbe.getStattype().getResult()).getYAxisDescription(), values, false,
						currentProbe.getStattype().getResult(), currentProbe.getDofs());
				resultContainer.addResult(r);
			}

			else if (currentProbe.getStattype().getResult() instanceof ContinuousSeriesResult) {

				EList<SeriesResultElement> list = ((ContinuousSeriesResult) currentProbe.getStattype().getResult()).getElements();
				List<Point> values = new ArrayList<Point>();
				for (SeriesResultElement a : list) {
					values.add(new Point(a.getX().doubleValue(), a.getY().doubleValue(), a.getConfidenceInterval()));
				}


				
				r = new SeriesResultPavo(resourcesToProbes.get(j), metricName, metricDescription.get(j),
						currentProbe.getStattype().getTypeName(),
						((AbstractSeriesResult) currentProbe.getStattype().getResult()).getXAxisDescription(),
						((AbstractSeriesResult) currentProbe.getStattype().getResult()).getYAxisDescription(), values, true,
						currentProbe.getStattype().getResult(), currentProbe.getDofs());
				resultContainer.addResult(r);
				 
			} else if (currentProbe.getStattype().getResult() instanceof ResultList<?>) {
				int i = 1;
				for (Result c : ((ResultList<?>) currentProbe.getStattype().getResult()).getResults()) {
					if(c instanceof DecimalResult)
					{
						resultContainer.addResult(
								new ValueResultPavo(resourcesToProbes.get(j) + "_" + i, metricName,
										metricDescription.get(j)
								, currentProbe.getStattype().getTypeName(), "", "",
										((DecimalResult) c).getValue().doubleValue(), currentProbe.getDofs()));
					}
					else
					{
					
					EList<SeriesResultElement> list = ((ContinuousSeriesResult) c).getElements();
					List<Point> values = new ArrayList<Point>();
					for (SeriesResultElement a : list) {
						values.add(new Point(a.getX().doubleValue(), a.getY().doubleValue(), a.getConfidenceInterval()));
					}
					boolean continous = false;
					if(c instanceof ContinuousSeriesResult)
						continous = true;
						resultContainer
								.addResult(new SeriesResultPavo(resourcesToProbes.get(j) + " Series " + i,
										metricName, metricDescription.get(j),
										currentProbe.getStattype().getTypeName(), ((AbstractSeriesResult) c).getXAxisDescription(),
										((AbstractSeriesResult) c).getYAxisDescription(), values, continous, c,
										currentProbe.getDofs()));
					i++;
					}
				}

			
						

			} else if (currentProbe.getStattype().getResult() instanceof OnlineResult) {
				
				r = new SeriesResultPavo(resourcesToProbes.get(j), metricName, metricDescription.get(j),
						currentProbe.getStattype().getTypeName(),
						((OnlineResult) currentProbe.getStattype().getResult()).getXAxisDescription(),
						((OnlineResult) currentProbe.getStattype().getResult()).getYAxisDescription(), new LinkedList<Point>(), true,
						((OnlineResult) currentProbe.getStattype().getResult()).getResult(), currentProbe.getDofs());
				resultContainer.addResult(r);
				
			}
			j++;
		}
		return resultContainer;

	}

	}

