
/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pavo.result;

import java.util.List;

import tools.descartes.pavo.util.dof;

/**
 * class for a single value as result for pavo
 * 
 * @author Maximilian König <max.c.koenig@t-online.de>
 * @since 2016
 */
public class ValueResultPavo extends AbstractQuantitativeResult {

	private Double value;

	private List<dof> dofs = null;

	/**
	 * @param queryableElementN
	 *            current queryable element
	 * @param metricN
	 *            current metric
	 * @param statisticTypeN
	 *            current StatisticType
	 * @param valueN
	 *            Value to be shown in pavo
	 */
	public ValueResultPavo(String queryableElementN, String metricN, String description, String statisticTypeN,
			String scalexN,
			String scaleyN, Double valueN, List<dof> dofsN) {
		super(queryableElementN, metricN, description, statisticTypeN, scalexN, scaleyN, null);
		value = valueN;
		dofs = dofsN;
	}

	/**
	 * @return the value
	 */
	public Double getValue() {
		return value;
	}

	/**
	 * @param value
	 *            the value to set
	 */
	public void setValue(Double value) {
		this.value = value;
	}

	public String getName() {
		if (additionalDescription == null || additionalDescription.equals(null))
			additionalDescription = "";
		if (!this.getStatisticType().equals("DEFAULT"))
			return this.getQueryableElement() + " " + this.getStatisticType() + this.additionalDescription;
		else
			return this.getQueryableElement() + this.additionalDescription;
	}

	public List<dof> getDofs() {
		return dofs;
	}

	public void setDofs(List<dof> dofs) {
		this.dofs = dofs;
	}

}
