/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
/**
 * This interface declares all different kinds of Diagrams, which can be shown by pavo
 * 
 * @author Maximilian Koenig
 */
package tools.descartes.pavo.types;

import tools.descartes.pavo.result.ResultContainerPavo;

/**
 * Abstract class for all different kinds of visualizations. The PAVO result
 * container is taken as attribute.
 * 
 * @author Maximilian König <max.c.koenig@t-online.de>
 * @since 2016
 */
public abstract class AbstractVisualizationType implements IVisualizationType {

	private ResultContainerPavo result;

	public AbstractVisualizationType() {
	}

	public AbstractVisualizationType(ResultContainerPavo resultN) {
		result = resultN;
	}

	public String getXDescription() {
		String description = "";
		String metric = this.getResult().getMetric();
		if (metric.contains("time") | metric.contains("Time")) {
			if (this.getResult().getScalex().contains("arrivals")) {
				description = "experiment arrivals";
			} else {
				// description = "arrivals" + " in " + "ms";
				description = "experiment time" + " in " + "ms";
			}
		} else if (metric.contains("utilization") | metric.contains("Utilization")) {
			// description = "measurement time" + " in " + "ms";
			description = "experiment time" + " in " + "ms";
		} else {
			description = "";
		}
		return description;
	}

	public String getYDescription() {
		if (this.getResult().getMetric() != null) {
			String metric = this.getResult().getMetric();
			metric = metric.replace("avg", "");
			metric = metric.replace("average", "");
			if (this.getResult().getScaley() != null && this.getResult().getScaley() != "") {
				return metric + " in " + this.getResult().getScaley();
			} else {
				return metric;
			}
		}
		return "";
	}

	public ResultContainerPavo getResult() {
		return result;
	}

	public void setResult(ResultContainerPavo result) {
		this.result = result;
	}
}