/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package tools.descartes.pavo.types;

import java.awt.Color;

import javax.swing.JPanel;
import javax.swing.JRadioButton;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.data.general.DefaultPieDataset;

import tools.descartes.pavo.controller.VisualizationController;
import tools.descartes.pavo.result.AbstractQuantitativeResult;
import tools.descartes.pavo.result.ResultContainerPavo;
import tools.descartes.pavo.result.SeriesResultPavo;
import tools.descartes.pavo.result.ValueResultPavo;
import tools.descartes.pavo.util.Point;
import tools.descartes.pavo.view.View;

/**
 * Class to visualise the current result as PieChart if possible. Can visualise
 * results constiting out of value results, which are 100 or 1.0 added. Does not
 * do any configurations for the control buttons
 * 
 * @author Maximilian König <max.c.koenig@t-online.de>
 * @since 2016
 */
public class PieChart extends AbstractVisualizationType implements IVisualizationType {

	public PieChart(ResultContainerPavo resultN) {
		super(resultN);

	}

	@Override
	public JFreeChart visualize() {
		DefaultPieDataset dataset = new DefaultPieDataset();
		if (this.getResult().getResults().size() == 1
				&& this.getResult().getResults().get(0) instanceof SeriesResultPavo) {
			for (Point p : ((SeriesResultPavo) this.getResult().getResults().get(0)).getValues()) {
				dataset.setValue(this.getResult().getResults().get(0).getMetric(), p.getY());
			}
		} else {
			int i = 1;
			for (AbstractQuantitativeResult r : this.getResult().getResults()) {

				dataset.setValue(r.getMetric() + " " + i, ((ValueResultPavo) r).getValue());
				i++;
			}

		}
		JFreeChart chart = ChartFactory.createPieChart("", // chart title
				dataset, // data
				true, // include legend
				true, false);
		chart.setBackgroundPaint(Color.WHITE);
		chart.setBorderPaint(Color.WHITE);
		return chart;
	}

	@Override
	public boolean canVisualize() {
		int fullValue = 0;
		if (this.getResult().getResults().size() == 1
				&& this.getResult().getResults().get(0) instanceof SeriesResultPavo) {
			for (Point p : ((SeriesResultPavo) this.getResult().getResults().get(0)).getValues()) {
				fullValue += p.getY();
			}
		} else {
			for (AbstractQuantitativeResult r : this.getResult().getResults()) {
				if (r instanceof ValueResultPavo) {
					fullValue += ((ValueResultPavo) r).getValue();
				}
			}
		}
		if (fullValue == 1 || fullValue == 100) {
			return true;
		}
		return false;
	}

	public JPanel getEditViewComponent(VisualizationController c) {
		JPanel panel = new JPanel();
		JRadioButton a = View.createConfidence(c.getActionListenerConfidence());
		a.setEnabled(false);
		panel.add(a);
		JPanel b = View.createMinMaxMean(c.createActionListenerMin(), c.createActionListenerMax(),
				c.createActionListenerMean());
		b.setEnabled(false);
		panel.add(b);
		return panel;
	}

}